"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var chalk = require("chalk");
var path = require("path");
var Line = /** @class */ (function () {
    function Line(num, content) {
        this.numbering = num;
        this.content = content;
    }
    return Line;
}());
var Lines = /** @class */ (function (_super) {
    __extends(Lines, _super);
    function Lines(lines) {
        var _this = _super.call(this) || this;
        if (lines && lines instanceof Array) {
            lines.forEach(function (line) { return _this.push(line); });
        }
        Object.setPrototypeOf(_this, Object.create(Lines.prototype));
        return _this;
    }
    Lines.prototype.getContent = function () {
        return this.map(function (line) { return line.content; }).join("\n");
    };
    Lines.prototype.findLines = function (lineContent) {
        return new Lines(this.filter(function (line) { return line.content.includes(lineContent); }));
    };
    Lines.prototype.findLine = function (num) {
        return this[num - 1];
    };
    Lines.prototype.getBlockAroundLine = function (lineNumber, blockSize) {
        if (blockSize % 2 !== 1) {
            throw new Error("Block Size Must Be ");
        }
        if (lineNumber < 1) {
            throw new Error("Invalid Line Number");
        }
        blockSize = Math.ceil(blockSize / 2);
        var startPoint = lineNumber - blockSize;
        var endPoint = lineNumber + blockSize;
        var block = new Lines();
        for (var i = startPoint; i <= endPoint; i++) {
            var line = this.findLine(i) || new Line(undefined, "");
            if (line) {
                block.push(line);
            }
        }
        return block;
    };
    return Lines;
}(Array));
var FileController = /** @class */ (function () {
    function FileController(path, content) {
        this._path = path;
        var fileContent = content;
        if (!content)
            fileContent = fs.readFileSync(this._path, { encoding: 'utf8', flag: 'r' });
        this._content = new Lines(fileContent.split("\n").map(function (line, index) { return new Line(index + 1, line); }));
    }
    Object.defineProperty(FileController.prototype, "content", {
        get: function () {
            return this._content.getContent();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FileController.prototype, "path", {
        get: function () {
            return path.resolve(this._path);
        },
        enumerable: false,
        configurable: true
    });
    FileController.prototype.formattedLineVisual = function (lineContent, last, disableColor, headerFormatter) {
        if (last === void 0) { last = true; }
        if (disableColor === void 0) { disableColor = false; }
        var lines = this._content.findLines(lineContent);
        if (last) {
            lines.reverse();
        }
        if (lines.length === 0) {
            return headerFormatter ? headerFormatter(this) : '';
        }
        var line = lines[0];
        var block = this._content.getBlockAroundLine(line.numbering, 5);
        var formratedString = block.reduce(function (prev, curr) {
            var isSelectedLine = curr.numbering === line.numbering;
            var formatted = "" + prev + (isSelectedLine ? '> ' : '  ') + " " + (curr.numbering ? curr.numbering + ")" : ' ') + " " + curr.content + "\n";
            return isSelectedLine ? !disableColor ? chalk.red(formatted) : formatted : !disableColor ? chalk.white(formatted) : formatted;
        }, '\n');
        return "" + (headerFormatter ? headerFormatter(this) : '') + formratedString;
    };
    return FileController;
}());
exports.default = FileController;
//# sourceMappingURL=file.js.map