"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./../src/utils");
var example_blocks_1 = require("./examples/example-blocks");
var block_1 = require("../src/block");
var example_info_1 = require("./examples/example-info");
require("./__mocks__/fs");
jest.mock('fs');
var fs = require("fs");
describe("Block Class", function () {
    test("Creation", function () {
        var type = "Type-X", name = "Name-Y", content = "Content-Z";
        var block = new block_1.Block(type, name, content);
        expect(block.type).toBe(type);
        expect(block.name).toBe(name);
        expect(block.content).toBe(content);
    });
});
describe("File Info Class", function () {
    test("Creation", function () {
        var info = new block_1.ExtendInfo();
        expect(info.isExtending).toBe(false);
        expect(info.errors).toHaveLength(0);
        expect(info.fileModifiedContent).toBe('');
        expect(info.extendedFile).toBe('');
    });
});
describe("Blocks Controller", function () {
    describe("GetExtendingInfo", function () {
        test("extending", function () {
            var info = block_1.BlockController.getExtendingInfo({ content: example_info_1.FileExtendingExample.content });
            expect(info.isExtending).toBe(true);
            expect(info.errors).toHaveLength(0);
            expect(info.fileModifiedContent).toBe(example_info_1.FileExtendingExample.after);
            expect(info.extendedFile).toBe('.' + example_info_1.FileExtendingExample.extending);
        });
        test("not extending", function () {
            var info = block_1.BlockController.getExtendingInfo({ content: example_info_1.FileNotExtendingExample.content });
            expect(info.isExtending).toBe(false);
            expect(info.errors).toHaveLength(0);
            expect(info.fileModifiedContent).toBe(example_info_1.FileNotExtendingExample.content);
            expect(info.extendedFile).toBe('');
        });
        test("extend multiple files", function () {
            var extend = function () { return block_1.BlockController.getExtendingInfo({ content: example_info_1.FileMultiExtendingExample.content }); };
            expect(extend).toThrowError();
        });
    });
    describe("ConvertToBlocks", function () {
        test("one level & one block", function () {
            var blockLevels = block_1.BlockController.ConvertToBlocks({ content: example_blocks_1.FileWithOneBlock });
            expect(blockLevels).toHaveLength(1);
            expect(blockLevels[0]).toHaveLength(3);
            expect(blockLevels[0].filter(function (block) { return block.type === "none"; })).toHaveLength(2);
        });
        test("one level & multipe blocks", function () {
            var blockLevels = block_1.BlockController.ConvertToBlocks({ content: example_blocks_1.FileWithMultiBlock });
            expect(blockLevels).toHaveLength(1);
            expect(blockLevels[0]).toHaveLength(7);
            expect(blockLevels[0].filter(function (block) { return block.type === "none"; })).toHaveLength(4);
        });
        test("Two levels & multipe blocks", function () {
            fs.__setFileContent(example_blocks_1.FileTwoLevelsBlocks.parent);
            var blockLevels = block_1.BlockController.ConvertToBlocks({ content: example_blocks_1.FileTwoLevelsBlocks.child });
            expect(blockLevels).toHaveLength(2);
            expect(blockLevels[0]).toHaveLength(5);
            expect(blockLevels[0].filter(function (block) { return block.type === "none"; })).toHaveLength(3);
            expect(blockLevels[1]).toHaveLength(7);
            expect(blockLevels[1].filter(function (block) { return block.type === "none"; })).toHaveLength(4);
        });
    });
    describe("Merging Blocks", function () {
        //to be done
    });
});
describe("Patterns Test", function () {
    describe("Extend Expression Pattern Test", function () {
        test("Basic usage test ", function () {
            var example = "/* @extends 'file' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Usage with double quotes", function () {
            var example = "/* @extends \"file\" */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Usage without quotes", function () {
            var example = "/* @extends file */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(false);
        });
        test("Usage with wrong spelling", function () {
            var example = "/* @extend 'file' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(false);
        });
        test("Against file naming with hyphen", function () {
            var example = "/* @extends 'file-file' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Against file naming with underscore", function () {
            var example = "/* @extends 'file_file' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Against file naming not english", function () {
            var example = "/* @extends 'عربي' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Against file naming nested paths", function () {
            var example = "/* @extends 'folder/file.ext' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Against file naming nested paths more levels", function () {
            var example = "/* @extends 'folder/folder/file.ext' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Against file naming nested paths relative ", function () {
            var example = "/* @extends './folder/folder/file.ext' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Against file naming nested paths backwards ", function () {
            var example = "/* @extends '../../folder/folder/file.ext' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(true);
        });
        test("Against file with no name", function () {
            var example = "/* @extends '' */";
            expect(utils_1.Patterns.extendRegex.test(example)).toBe(false);
        });
    });
    describe("Block Header Expression Pattern Test", function () {
        test("Basic Usage", function () {
            var example = "/* @block someblock */";
            expect(utils_1.Patterns.blockHeaderRegex.test(example)).toBe(true);
        });
        test("Naming with hyphen", function () {
            var example = "/* @block some-block */";
            expect(utils_1.Patterns.blockHeaderRegex.test(example)).toBe(true);
        });
        test("Naming with underscore", function () {
            var example = "/* @block some_block */";
            expect(utils_1.Patterns.blockHeaderRegex.test(example)).toBe(true);
        });
        test("Naming with another lang", function () {
            var example = "/* @block عربي */";
            expect(utils_1.Patterns.blockHeaderRegex.test(example)).toBe(true);
        });
        test("Adding prepend block", function () {
            var example = "/* @block prepend someblock */";
            expect(utils_1.Patterns.blockHeaderRegex.test(example)).toBe(true);
        });
        test("Adding append block", function () {
            var example = "/* @block append someblock */";
            expect(utils_1.Patterns.blockHeaderRegex.test(example)).toBe(true);
        });
        test("Empty name", function () {
            var example = "/* @block  */";
            expect(utils_1.Patterns.blockHeaderRegex.test(example)).toBe(false);
        });
    });
    describe("Terminate Block Expression Pattern Test", function () {
        test("Basic Usage", function () {
            var example = "/* @terminate block */";
            expect(utils_1.Patterns.blockFooterRegex.test(example)).toBe(true);
        });
        test("Wrong Spelling ", function () {
            var example = "/* @terminat block */";
            expect(utils_1.Patterns.blockFooterRegex.test(example)).toBe(false);
        });
    });
});
//# sourceMappingURL=block.test.js.map